#ifndef __CREEP_EVOLUTION_HH__
#define __CREEP_EVOLUTION_HH__

float compute_laplacian(const float cur,
			const float n1, const float n2, const float n3, const float n4)
{
  float output = n1 + n2 + n3 + n4 - 4*cur;
  return output;
}


void perform_one_step_creep_evolution(env_t& e, const int thread_id)
{
  image<float> & height_landscape		 = *(e.height_landscape);
  image<float> & internal_variation_landscape	 = *(e.internal_variation_landscape);
  const float DT = e.DT;
  const float EROSION_CREEP = e.EROSION_CREEP;

//   const int NB_W = 4;
//   const int win_i[NB_W] = {0,  0, -1, 1};
//   const int win_j[NB_W] = {-1, 1,  0, 0};

  const int begin_row = e.NR_threads[thread_id];
  const int end_row   = e.NR_threads[thread_id+1];
  const int NC		= height_landscape.ncols();

  // internal_variation_landscape is an image used internally

  for (int i=begin_row; i<end_row; ++i)
    {
      for (int j=0; j<NC; ++j)
	{
	  float creep = compute_laplacian(height_landscape(i,j),
					  height_landscape(i,j-1), height_landscape(i,j+1),
					  height_landscape(i-1,j), height_landscape(i+1,j)
					  );
	  creep = EROSION_CREEP * creep;
	  internal_variation_landscape(i,j) = creep;
	} // end for j
    } // end for i

#pragma omp barrier
  for (int i=begin_row; i<end_row; ++i)
    for (int j=0; j<NC; ++j)
      height_landscape(i,j) += DT * internal_variation_landscape(i,j) ;
#pragma omp barrier

  return ;
}

void perform_creep_evolution(env_t &e, const unsigned int thread_id)
{
   set_border_condition_ocean(e, *(e.level_water), *(e.height_landscape),
			      e.CST_OCEAN_LEVEL, thread_id);
  // perform one step of landscape evolution
   perform_one_step_creep_evolution(e, thread_id);
   return ;
}



#endif
