
#ifndef __ENV_HH__
#define __ENV_HH__

#include "image.hh"



struct env_s
{
  int NR;
  int NC;
  float DT;
  float RAIN;
  int NB_THREADS;
  int* NR_threads;

  float EROSION_EXP_M;
  float EROSION_E;
  float EROSION_CREEP;
  float EROSION_S;
  float CST_OCEAN_LEVEL;

  image<float> *level_water;
  image<float> *height_landscape;
//   image<float> *sediment;
  image<float> *water_times_concentration;
  image<float> *variation_landscape;

  image<float> *delta_water;
  image<float> *delta_water_times_concentration;
  image<float> *internal_variation_landscape;

};

typedef struct env_s env_t;



void create_env(env_t & e,
		float DT,
		const int NR,
		const int NC,
		const unsigned int NB_THREADS,
		float RAIN,
		float EROSION_EXP_M,
		float EROSION_E,
		float EROSION_CREEP,
		float EROSION_S,
		float CST_OCEAN_LEVEL
		)
{
  e.NR = NR;
  e.NC = NC;
  e.DT = DT;
  e.NB_THREADS = NB_THREADS;
  e.NR_threads = new int[NB_THREADS+1];
  for (int i=0;i<e.NB_THREADS; ++i)
    e.NR_threads[i] = i*e.NR/e.NB_THREADS;
  e.NR_threads[e.NB_THREADS] = e.NR;

  e.RAIN = RAIN;

  e.EROSION_EXP_M = EROSION_EXP_M;
  e.EROSION_E = EROSION_E;
  e.EROSION_CREEP = EROSION_CREEP;
  e.EROSION_S = EROSION_S;
  e.CST_OCEAN_LEVEL = CST_OCEAN_LEVEL;

  e.height_landscape		= new image<float>(NR, NC);
  e.level_water			= new image<float>(NR, NC);
//   e.sediment			= new image<float>(NR, NC);
  e.water_times_concentration	= new image<float>(NR, NC);

  e.variation_landscape		= new image<float>(NR, NC);


  e.internal_variation_landscape= new image<float>(NR, NC);
  e.delta_water			= new image<float>(NR, NC);
  e.delta_water_times_concentration = new image<float>(NR, NC);
}


void destroy_env(env_t & e)
{
  delete e.delta_water_times_concentration;
  delete e.internal_variation_landscape;
  delete e.delta_water;

  delete e.variation_landscape;
  delete e.water_times_concentration;
//   delete e.sediment;
  delete e.level_water;
  delete e.height_landscape;
}



#endif
