#ifndef __IMAGE_HH__
#define __IMAGE_HH__

#include <assert.h>
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <string.h>


template<class D>
struct image
{
  image(const int nr, const int nc, const int BORDER = 4) :
    nrows_(nr), ncols_(nc), nborders_(BORDER)

  {
    const int npoints = (nrows_+2*nborders_) * (ncols_+2*nborders_);
    data = (D*)malloc(npoints *sizeof(D));
    data += nborders_*(ncols_+2*nborders_) + nborders_;
  }


  image(const char *name, const D border_value = 0)
  {
    nborders_ = 0;
    {
      std::ifstream is(name, std::ios::in | std::ios::binary);
      if (! is.is_open())
        { std::cerr << "Error opening file"<<std::endl;exit (1); }

      char c = is.get();
      assert(c == 'P');
      c = is.get();
      assert(c == '5');
      c = is.get();
       assert(c == '\n');
      while (is.peek() == '#')
        {
          std::string line;
          std::getline(is,line);
        }
      {
        is >> ncols_;
        is >> nrows_;
        //skip max value
        int b;
        is >> b;
      }
      {
        char c;
        is.read(&c, 1);
         assert(c == '\n');
      }
      const int npoints = (nrows_) * (ncols_);
      data = (D*)malloc(npoints *sizeof(D));
      memset(data, border_value, npoints*sizeof(D));

      // Now read data;
      for (int i=0; i < nrows(); ++i)
        for (int j=0; j < ncols(); ++j)
          {
            int p = i * ncols() + j;
            char c;
            is.read(&c, 1);
            if (c < 0)
              data[p] = 256+int(c);
            else
              data[p] = c;
          }
      is.close();
    }
  }

  void save(const char *name)
  {
    FILE *fp = fopen(name, "w");
    fprintf(fp,"P5\n%d %d\n255\n",ncols(), nrows());
    fwrite(data, sizeof(unsigned char), nrows()*ncols(), fp);
    fclose(fp);
  }



  D& operator[](const int p)
  { return data[p];}

  D operator[](const int p) const
  { return data[p];}

  D& operator()(const int i, const int j)
  { return data[i * (ncols_+2*nborders_) + j];}

  D operator()(const int i, const int j) const
  { return data[i * (ncols_+2*nborders_) + j];}

  int to_index(const int i, const int j) const
  { return i * (ncols_+2*nborders_) + j;}


  int nrows() const {return nrows_;}
  int ncols() const {return ncols_;}
  int nborders() const {return nborders_;}

   ~image()
   {
    data -= nborders_*(ncols_+2*nborders_) + nborders_;
    free(data);
   }

  void del()
  {
    data -= nborders_*(ncols_+2*nborders_) + nborders_;
    free(data);
  }

  int nrows_;
  int ncols_;
  int nborders_;
  D *data;
};



template <class D>
void initialize_image(image<D>& ima,
                      const D init_value,
                      const D )
{
  for (int i=0; i<ima.nrows(); ++i)
    for (int j=0; j<ima.ncols(); ++j)
      {
        int p = i * ima.ncols() + j;
        ima[p] = init_value;
      }
}

#endif
