
#ifndef __SAVE_CC__
#define __SAVE_CC__

#include <string>
#include "image.hh"


  // generate gif for landscape
void generate_script_for_gif(const std::string & list_files,
			     const std::string & name_gif,
			     const std::string & name_script_file)
  {
    std::string command =
      std::string("convert ") + list_files + name_gif;
    FILE *fp = fopen(name_script_file.c_str(), "w");
    fprintf(fp,"%s\n",command.c_str());;
    fclose(fp);
  }


std::string int2string(int i)
{
  std::stringstream out;
  out << i;
  return  out.str();
}


void save_image(image<float>& ima, const std::string &name, float scale = 1)
{
  FILE *fp = fopen(name.c_str(), "w");
  fprintf(fp,"P5\n%d %d\n255\n", ima.ncols(), ima.nrows());

  unsigned char *data = (unsigned char*)malloc(ima.ncols()*sizeof(unsigned char));

  for (int i=0; i<ima.nrows(); ++i)
    {
      for (int j=0; j<ima.ncols(); ++j)
	{
	  float tmp = ima(i,j);
	  tmp *= scale;
	  if (tmp<0) tmp = 0;
	  if (tmp>255) tmp = 255;
	  data[j] = (unsigned char)tmp;
	}
      fwrite(data, sizeof(unsigned char), ima.ncols(), fp);
    }
  fclose(fp);
  free(data);
}



void save_image_tiff(image<float>& ima, const std::string &name, float scale = 1)
{

   /* write image */
 /* allocate memory for data_out */
  int n_row = ima.nrows();
  int n_col = ima.ncols();
  int n_channels = 3;
  float * data_out;

  if (NULL == (data_out = (float *) malloc(n_channels* n_col * n_row * sizeof(float))))
    FATAL("error while reading the TIFF image");

  std::cout  << " SAVE------------------------------------------ " << std::endl;
      for (int i=0; i<n_row; ++i)
	for (int j=0; j<n_col; ++j)
	  {
	    float tmp = ima(i,j);
	    tmp *= scale;
	    if (tmp<0) tmp = 0;
	    if (tmp>255) tmp = 255;
	    float val = (unsigned char)tmp;
//   	    std::cout << val << " ";

	    data_out[i*n_col+j] = val; // rouge
	    data_out[i*n_col+j + n_row*n_col] = val; // vert
	    data_out[i*n_col+j + 2*(n_row*n_col)] = val; // bleu
	  }


      write_tiff_f32(name.c_str(), data_out, n_col, n_row, n_channels);
      free(data_out);




//   FILE *fp = fopen(name.c_str(), "w");
//   fprintf(fp,"P5\n%d %d\n255\n", ima.ncols(), ima.nrows());

//   unsigned char data[ima.ncols()];

//   for (int i=0; i<ima.nrows(); ++i)
//     {
//       for (int j=0; j<ima.ncols(); ++j)
// 	{
// 	  float tmp = ima(i,j);
// 	  tmp *= scale;
// 	  if (tmp<0) tmp = 0;
// 	  if (tmp>255) tmp = 255;
// 	  data[j] = (unsigned char)tmp;
// 	}
//       fwrite(data, sizeof(unsigned char), ima.ncols(), fp);
//     }
//   fclose(fp);

}


void save_image_tiff_double(image<float>& ima, const std::string &name, float scale = 1)
{

   /* write image */
 /* allocate memory for data_out */
  int n_row = 2*ima.nrows();
  int n_col = 2*ima.ncols();
  int n_channels = 3;
  float * data_out;

  if (NULL == (data_out = (float *) malloc(n_channels* n_col * n_row * sizeof(float))))
    FATAL("error while reading the TIFF image");

  std::cout  << " SAVE------------------------------------------ " << std::endl;
      for (int i=0; i<n_row; ++i)
	for (int j=0; j<n_col; ++j)
	  {
	    float tmp = ima(i/2,j/2);
	    tmp *= scale;
// 	    if (tmp<0) tmp = 0;
// 	    if (tmp>255) tmp = 255;
// 	    float val = (unsigned char)tmp;
//   	    std::cout << val << " ";

	    if ((tmp >= 0) && (tmp <= 255))
	      {
		float val = tmp;
		data_out[i*n_col+j] = val; // rouge
		data_out[i*n_col+j + n_row*n_col] = val; // vert
		data_out[i*n_col+j + 2*(n_row*n_col)] = val; // bleu
	      }
	    else
	      if (tmp < 0)
		{
		  data_out[i*n_col+j] = 0; // rouge
		  data_out[i*n_col+j + n_row*n_col] = 0; // vert
		  data_out[i*n_col+j + 2*(n_row*n_col)] = 255; // bleu
		}
	      else
		{
		  data_out[i*n_col+j] = 255; // rouge
		  data_out[i*n_col+j + n_row*n_col] = 0; // vert
		  data_out[i*n_col+j + 2*(n_row*n_col)] = 0; // bleu
		}

	  }


      write_tiff_f32(name.c_str(), data_out, n_col, n_row, n_channels);
      free(data_out);


}



void save_image_tiff_quad_saturated(image<float>& ima, const std::string &name, float scale = 1)
{

   /* write image */
 /* allocate memory for data_out */
  int n_row = 4*ima.nrows();
  int n_col = 4*ima.ncols();
  int n_channels = 3;
  float * data_out;

  if (NULL == (data_out = (float *) malloc(n_channels* n_col * n_row * sizeof(float))))
    FATAL("error while reading the TIFF image");

  std::cout  << " SAVE------------------------------------------ " << std::endl;
      for (int i=0; i<n_row; ++i)
	for (int j=0; j<n_col; ++j)
	  {
	    float tmp = ima(i/4,j/4);
	    tmp *= scale;
// 	    if (tmp<0) tmp = 0;
// 	    if (tmp>255) tmp = 255;
// 	    float val = (unsigned char)tmp;
//   	    std::cout << val << " ";

	    if ((tmp >= 0) && (tmp <= 255))
	      {
		float val = tmp;
		data_out[i*n_col+j] = val; // rouge
		data_out[i*n_col+j + n_row*n_col] = val; // vert
		data_out[i*n_col+j + 2*(n_row*n_col)] = val; // bleu
	      }
	    else
	      if (tmp < 0)
		{
		  data_out[i*n_col+j] = 0; // rouge
		  data_out[i*n_col+j + n_row*n_col] = 0; // vert
		  data_out[i*n_col+j + 2*(n_row*n_col)] = 0; // bleu
		}
	      else
		{
		  data_out[i*n_col+j] = 255; // rouge
		  data_out[i*n_col+j + n_row*n_col] = 255; // vert
		  data_out[i*n_col+j + 2*(n_row*n_col)] = 255; // bleu
		}

	  }


      write_tiff_f32(name.c_str(), data_out, n_col, n_row, n_channels);
      free(data_out);


}












void save_image_color(image<float>& ima, const std::string &name, float scale = 1)
{
  FILE *fp = fopen(name.c_str(), "w");
  fprintf(fp,"P6\n%d %d\n255\n", ima.ncols(), ima.nrows());

  unsigned char *data = (unsigned char*)malloc(3*ima.ncols()*sizeof(unsigned char));

  for (int i=0; i<ima.nrows(); ++i)
    {
      for (int j=0; j<ima.ncols(); ++j)
	{
	  float tmp = ima(i,j);
	  tmp *= scale;
	  data[3*j+1] = 0*(unsigned char)tmp;
	  data[3*j+2] = 0*(unsigned char)tmp;
	  data[3*j+3] = 255;//(unsigned char)tmp;
	  if (tmp<0)
	    {
	      data[3*j+1] = 0;
	      data[3*j+2] = 255;//blue
	      data[3*j+3] = 0;
	    }
	  if (tmp>255)
	    {
	      data[3*j+1] = 0;
	      data[3*j+2] = 0;
	      data[3*j+3] = 255; //red
	    }

	}      fwrite(data, 3*sizeof(unsigned char), ima.ncols(), fp);
    }
  fclose(fp);
  free(data);
}







void save_image_tiff_quad_saturated_visa_water(image<float>& ima, const std::string &name,
					       float OCEAN_LEVEL)
{

   /* write image */
 /* allocate memory for data_out */
  int n_row = 4*ima.nrows();
  int n_col = 4*ima.ncols();
  int n_channels = 3;
  float * data_out;

  if (NULL == (data_out = (float *) malloc(n_channels* n_col * n_row * sizeof(float))))
    FATAL("error while reading the TIFF image");

  float mean_above_ocean_level = 0;
  for (int i=0; i<ima.nrows(); ++i)
    for (int j=0; j<ima.ncols(); ++j)
      mean_above_ocean_level += fmax(0, ima(i,j) - OCEAN_LEVEL);
  mean_above_ocean_level = mean_above_ocean_level/(ima.nrows()*ima.ncols());
  float cst_scale = 64.0/(OCEAN_LEVEL + mean_above_ocean_level+0.0001);



  std::cout  << " SAVE------------------------------------------ " << std::endl;
      for (int i=0; i<n_row; ++i)
	for (int j=0; j<n_col; ++j)
	  {
	    float tmp = ima(i/4,j/4);
	    tmp *= cst_scale;
	    if ((tmp >= 0) && (tmp <= 255))
	      {
		float val = tmp;
		data_out[i*n_col+j] = val; // rouge
		data_out[i*n_col+j + n_row*n_col] = val; // vert
		data_out[i*n_col+j + 2*(n_row*n_col)] = val; // bleu
	      }
	    else
	      if (tmp < 0)
		{
		  data_out[i*n_col+j] = 0; // rouge
		  data_out[i*n_col+j + n_row*n_col] = 0; // vert
		  data_out[i*n_col+j + 2*(n_row*n_col)] = 0; // bleu
		}
	      else
		{
		  data_out[i*n_col+j] = 255; // rouge
		  data_out[i*n_col+j + n_row*n_col] = 255; // vert
		  data_out[i*n_col+j + 2*(n_row*n_col)] = 255; // bleu
		}

	  }


      write_tiff_f32(name.c_str(), data_out, n_col, n_row, n_channels);
      free(data_out);


}



void save_image_tiff_quad_saturated_old_minus_new_landscape(image<float>& diff,
							    const std::string &name)
{

   /* write image */
 /* allocate memory for data_out */
  int n_row = 4*diff.nrows();
  int n_col = 4*diff.ncols();
  int n_channels = 3;
  float * data_out;

  if (NULL == (data_out = (float *) malloc(n_channels* n_col * n_row * sizeof(float))))
    FATAL("error while reading the TIFF image");

//   float mean_above_ocean_level = 0;
//   for (int i=0; i<ima.nrows(); ++i)
//     for (int j=0; j<ima.ncols(); ++j)
//       mean_above_ocean_level += fmax(0, ima(i,j) - OCEAN_LEVEL);
//   mean_above_ocean_level = mean_above_ocean_level/(ima.nrows()*ima.ncols());
//   float cst_scale = 64.0/(OCEAN_LEVEL + mean_above_ocean_level+0.0001);

  //compute mean
  float mean = 0;
  for (int i=0; i<diff.nrows(); ++i)
    for (int j=0; j<diff.ncols(); ++j)
      mean += diff(i,j);
  mean = mean / (diff.nrows()*diff.ncols());

  //compute variance
  float var = 0;
  for (int i=0; i<diff.nrows(); ++i)
    for (int j=0; j<diff.ncols(); ++j)
      var += (diff(i,j) - mean)*(diff(i,j) - mean);
  var = var / (diff.nrows()*diff.ncols()-1);
  float sigma = sqrt(var);

  float cst_scale = 1./(0.000001+2.*sigma);

//   std::cout  << " SAVE------------------------------------------ " << std::endl;
      for (int i=0; i<n_row; ++i)
	for (int j=0; j<n_col; ++j)
	  {
	    float tmp = diff(i/4,j/4) - mean;
	    tmp = tmp*cst_scale*128 + 128;
	    if ((tmp >= 0) && (tmp <= 255))
	      {
		float val = tmp;
		data_out[i*n_col+j] = val; // rouge
		data_out[i*n_col+j + n_row*n_col] = val; // vert
		data_out[i*n_col+j + 2*(n_row*n_col)] = val; // bleu
	      }
	    else
	      if (tmp < 0)
		{
		  data_out[i*n_col+j] = 0; // rouge
		  data_out[i*n_col+j + n_row*n_col] = 0; // vert
		  data_out[i*n_col+j + 2*(n_row*n_col)] = 0; // bleu
		}
	      else
		{
		  data_out[i*n_col+j] = 255; // rouge
		  data_out[i*n_col+j + n_row*n_col] = 255; // vert
		  data_out[i*n_col+j + 2*(n_row*n_col)] = 255; // bleu
		}

	  }


      write_tiff_f32(name.c_str(), data_out, n_col, n_row, n_channels);
      free(data_out);
}







void save_image_tiff_quad_saturated_concentration(image<float>& concentration,
						  const std::string &name)
{
  /* write image */
  /* allocate memory for data_out */
  int n_row = 4*concentration.nrows();
  int n_col = 4*concentration.ncols();
  int n_channels = 3;
  float * data_out;

  if (NULL == (data_out = (float *) malloc(n_channels* n_col * n_row * sizeof(float))))
    FATAL("error while reading the TIFF image");

//   float mean_above_ocean_level = 0;
//   for (int i=0; i<ima.nrows(); ++i)
//     for (int j=0; j<ima.ncols(); ++j)
//       mean_above_ocean_level += fmax(0, ima(i,j) - OCEAN_LEVEL);
//   mean_above_ocean_level = mean_above_ocean_level/(ima.nrows()*ima.ncols());
//   float cst_scale = 64.0/(OCEAN_LEVEL + mean_above_ocean_level+0.0001);

  //compute mean
  float mean = 0;
  for (int i=0; i<concentration.nrows(); ++i)
    for (int j=0; j<concentration.ncols(); ++j)
      mean += concentration(i,j);
  mean = mean / (concentration.nrows()*concentration.ncols());

  //compute variance
  float var = 0;
  for (int i=0; i<concentration.nrows(); ++i)
    for (int j=0; j<concentration.ncols(); ++j)
      var += (concentration(i,j) - mean)*(concentration(i,j) - mean);
  var = var / (concentration.nrows()*concentration.ncols()-1);
  float sigma = sqrt(var);

  float cst_scale = 1./(0.000001+2.*sigma);

//   std::cout  << " SAVE------------------------------------------ " << std::endl;
      for (int i=0; i<n_row; ++i)
	for (int j=0; j<n_col; ++j)
	  {
	    float tmp = concentration(i/4,j/4);
	    tmp = tmp*cst_scale*255;
	    if ((tmp >= 0) && (tmp <= 255))
	      {
		float val = tmp;
		data_out[i*n_col+j] = val; // rouge
		data_out[i*n_col+j + n_row*n_col] = val; // vert
		data_out[i*n_col+j + 2*(n_row*n_col)] = val; // bleu
	      }
	    else
	      if (tmp < 0)
		{
		  data_out[i*n_col+j] = 0; // rouge
		  data_out[i*n_col+j + n_row*n_col] = 0; // vert
		  data_out[i*n_col+j + 2*(n_row*n_col)] = 0; // bleu
		}
	      else
		{
		  data_out[i*n_col+j] = 255; // rouge
		  data_out[i*n_col+j + n_row*n_col] = 255; // vert
		  data_out[i*n_col+j + 2*(n_row*n_col)] = 255; // bleu
		}

	  }


      write_tiff_f32(name.c_str(), data_out, n_col, n_row, n_channels);
      free(data_out);
}






#endif
