#ifndef __WATER_EVOLUTION_HH__
#define __WATER_EVOLUTION_HH__

#include <cmath>

#include "image.hh"
#include "border_condition.cc"


// current site s
// neighboring site t
inline
float get_variation_water_level_for_current_site(const float landscape_s,
						 const float water_s,
						 const float landscape_t,
						 const float water_t)
{
  float steep = (landscape_t + water_t) -  (landscape_s + water_s);
  if (steep > 0) // t is higher
    return water_t * steep; // so s receives water
  return water_s * steep; // so s sends water
}

// return time step used during the iiteration
void perform_one_step_water_evolution(env_t &e, const unsigned int thread_id)
{
  image<float>& level_water       = *(e.level_water);
  image<float>& height_landscape  = *(e.height_landscape);
  image<float>& delta_water	  = *(e.delta_water);
  const float RAIN                = e.RAIN;

  const float DT                  = e.DT;

  //   const int NB_W = 4;
  //   const int win_i[NB_W] = {0,  0, -1, 1};
  //   const int win_j[NB_W] = {-1, 1,  0, 0};

  const int begin_row = e.NR_threads[thread_id];
  const int end_row   = e.NR_threads[thread_id+1];
  const int NC		= level_water.ncols();

  //   for (int i=begin_row; i<end_row; ++i)
  //     for (int j=0; j<NC; ++j)
  //       delta_water(i,j) = 0;

  // compute variations
  for (int i=begin_row; i<end_row; ++i)
    for (int j=0; j<NC; ++j)
      {
	float delta_water_cur = 0;

	delta_water_cur +=
	  get_variation_water_level_for_current_site(height_landscape(i,j),
						     level_water     (i,j),
						     height_landscape(i,j-1),
						     level_water     (i,j-1));
	delta_water_cur +=
	  get_variation_water_level_for_current_site(height_landscape(i,j),
						     level_water     (i,j),
						     height_landscape(i,j+1),
						     level_water     (i,j+1));
	delta_water_cur +=
	  get_variation_water_level_for_current_site(height_landscape(i,j),
						     level_water     (i,j),
						     height_landscape(i-1,j),
						     level_water     (i-1,j));
	delta_water_cur +=
	  get_variation_water_level_for_current_site(height_landscape(i,j),
						     level_water     (i,j),
						     height_landscape(i+1,j),
						     level_water     (i+1,j));
	delta_water(i,j) = delta_water_cur;
      }


#pragma omp barrier
  for (int i=begin_row; i<end_row; ++i)
    for (int j=0; j<NC; ++j)
      {
	level_water(i,j) += DT*(delta_water(i,j) + RAIN);
	//   	level_water(i,j) = fmax(level_water(i,j),0);
      }
#pragma omp barrier
}



void perform_water_evolution(env_t &e, const unsigned int thread_id)
{
  // set border condition for water
//   set_border_condition_dirichlet(e, *(e.level_water), thread_id);
  // set border condition for landscape
//   set_border_condition_laplacian_zero(e, *(e.height_landscape), thread_id);
//   set_border_condition_dirichlet(e, *(e.height_landscape), thread_id);


//    set_border_condition_laplacian_zero(e, *(e.level_water), thread_id);
//    set_border_condition_dirichlet(e, *(e.level_water), thread_id);

  set_border_condition_ocean(e, *(e.level_water), *(e.height_landscape),
			     e.CST_OCEAN_LEVEL, thread_id);

  // perform one step of water evolution
  perform_one_step_water_evolution(e, thread_id);
}

#endif
